@@ -9,6 +9,7 @@
 #include "cmdline_resolver.h"
 #include "cmdline_show.h"
 #include "cmdline_util.h"
+#include "cmdline_why.h"
 
 #include <ui.h>
 
@@ -17,6 +18,8 @@
 #include <generic/apt/download_signal_log.h>
 #include <generic/apt/infer_reason.h>
 
+#include <generic/util/util.h>
+
 #include <vscreen/fragment.h>
 #include <vscreen/vscreen.h>
 #include <vscreen/transcode.h>
@@ -273,6 +276,7 @@ static const char *cmdline_action_descriptions[num_pkg_action_states]={
   N_("The following NEW packages will be installed:"),
   N_("The following packages will be REMOVED:"),
   N_("The following packages will be upgraded:"),
+  N_("The following partially installed packages will be configured:")
 };
 
 // Probably something like cin.getline() would work, but I don't trust that
@@ -411,6 +415,14 @@ static bool prompt_trust()
 	  return true;
 	}
 
+      if(aptcfg->FindB("Apt::Get::AllowUnauthenticated", false))
+	{
+	  printf("%s",
+		 _("*** WARNING ***   Ignoring these trust violations because\n"
+		   "                  Apt::Get::AllowUnauthenticated is 'true'!\n"));
+	  return true;
+	}
+
 
       const string okstr=_("Yes"), abortstr=_("No");
 
@@ -590,29 +602,16 @@ static bool cmdline_show_preview(bool as_upgrade, pkgset &to_install,
 static void cmdline_parse_show(string response,
 			       int verbose)
 {
-  bool one_shown=false;
   // assume response[0]=='i'
-  string::size_type i=1;
+  std::vector<std::string> packages;
+  splitws(response, packages, 1, response.size());
 
-  while(i<response.size())
-    {
-      while(i<response.size() && isspace(response[i]))
-	++i;
-
-      string pkgname;
-      // Could support quoting, etc?
-      while(i<response.size() && !isspace(response[i]))
-	pkgname+=response[i++];
-
-      if(!pkgname.empty())
-	{
-	  one_shown=true;
-	  do_cmdline_show(pkgname, verbose);
-	}
-    }
-
-  if(!one_shown)
+  if(packages.empty())
     printf(_("No packages to show -- enter the package names on the line after 'i'.\n"));
+  else
+    for(std::vector<std::string>::const_iterator it = packages.begin();
+	it != packages.end(); ++it)
+      do_cmdline_show(*it, verbose);
 
   prompt_string(_("Press Return to continue."));
 }
@@ -620,24 +619,9 @@ static void cmdline_parse_show(string response,
 // Erm.  Merge w/ above?
 static void cmdline_parse_changelog(string response)
 {
-  // assume response[0]=='i'
-  string::size_type i=1;
-
   vector<string> packages;
-
-  while(i<response.size())
-    {
-      while(i<response.size() && isspace(response[i]))
-	++i;
-
-      string pkgname;
-      // Could support quoting, etc?
-      while(i<response.size() && !isspace(response[i]))
-	pkgname+=response[i++];
-
-      if(!pkgname.empty())
-	packages.push_back(pkgname);
-    }
+  // assume response[0]=='c'
+  splitws(response, packages, 1, response.size());
 
   if(packages.empty())
     printf(_("No packages found -- enter the package names on the line after 'c'.\n"));
@@ -647,6 +631,27 @@ static void cmdline_parse_changelog(string response)
   prompt_string(_("Press Return to continue"));
 }
 
+static void cmdline_parse_why(string response)
+{
+  vector<string> arguments;
+  // assume response[0]=='w'
+  splitws(response, arguments, 1, response.size());
+
+  if(arguments.empty())
+    printf(_("No packages found -- enter zero or more roots of the search followed by the package to justify."));
+  else
+    {
+      bool success;
+      string root = arguments.back();
+      arguments.pop_back();
+      std::auto_ptr<fragment> frag(do_why(arguments, root, false, false, success));
+      update_screen_width();
+      if(frag.get() != NULL)
+	std::cout << frag->layout(screen_width, screen_width, style());
+      _error->DumpErrors();
+    }
+}
+
 static inline fragment *flowindentbox(int i1, int irest, fragment *f)
 {
   return indentbox(i1, irest, flowbox(f));
@@ -662,6 +667,7 @@ static void prompt_help(ostream &out)
 				"d: %F"
 				"s: %F"
 				"v: %F"
+				"w: %F"
 				"e: %F"
 				"%n"
 				"%F"
@@ -673,6 +679,7 @@ static void prompt_help(ostream &out)
 				"%F"
 				"%F"
 				"%F"
+				"%F"
 				"%F"),
 			      flowindentbox(0, 3,
 					    fragf(_("continue with the installation"))),
@@ -688,6 +695,8 @@ static void prompt_help(ostream &out)
 					    fragf(_("toggle the display of changes in package sizes"))),
 			      flowindentbox(0, 3,
 					    fragf(_("toggle the display of version numbers"))),
+			      flowindentbox(0, 3,
+					    fragf(_("try to find a reason for installing a single package, or explain why installing one package should lead to installing another package."))),
 			      flowindentbox(0, 3,
 					    fragf(_("enter the full visual interface"))),
 			      flowbox(fragf(_("You may also specify modification to the actions which will be taken.  To do so, type an action character followed by one or more package names (or patterns).  The action will be applied to all the packages that you list.  The following actions are available:"))),
@@ -816,11 +825,15 @@ bool cmdline_do_prompt(bool as_upgrade,
 		case 'C':
 		  cmdline_parse_changelog(response);
 		  break;
+		case 'W': // should be 'Y' but that's for "yes"
+		  cmdline_parse_why(response);
+		  break;
 		case '+':
 		case '-':
 		case '=':
 		case '_':
 		case ':':
+		case '&':
 		  cmdline_parse_action(response, to_install, to_hold,
 				       to_remove, to_purge, verbose, true);
 		  break;
