@@ -1,6 +1,6 @@
 // apt.cc
 //
-//  Copyright 1999-2006 Daniel Burrows
+//  Copyright 1999-2007 Daniel Burrows
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -80,13 +80,13 @@ sigc::signal0<void> consume_errors;
 
 static void reset_interesting_dep_memoization()
 {
-  delete cached_deps_interesting;
+  delete[] cached_deps_interesting;
   cached_deps_interesting = NULL;
 }
 
 static void reset_surrounding_or_memoization()
 {
-  delete cached_surrounding_or;
+  delete[] cached_surrounding_or;
   cached_surrounding_or = NULL;
 }
 
@@ -289,6 +289,9 @@ void apt_load_cache(OpProgress *progress_bar, bool do_initselections,
 
   bool simulate = aptcfg->FindB(PACKAGE "::Simulate", false);
 
+  // Clear the error stack so that we don't get confused by old errors.
+  consume_errors();
+
   bool open_failed=!new_file->Open(*progress_bar, do_initselections,
 				   (getuid() == 0) && !simulate,
 				   status_fname)
@@ -386,7 +389,7 @@ pkg_action_state find_pkg_state(pkgCache::PkgIterator pkg)
 	    // FOO!  Should I abort here?
 	    return pkg_install;
 	}
-      else if(extstate.install_reason!=aptitudeDepCache::manual)
+      else if(state.Flags & pkgCache::Flag::Auto)
 	return pkg_auto_install;
       else
 	return pkg_install;
@@ -403,6 +406,10 @@ pkg_action_state find_pkg_state(pkgCache::PkgIterator pkg)
 
   else if(state.iFlags&pkgDepCache::ReInstall)
     return pkg_reinstall;
+  // States where --configure fixes things.
+  else if(pkg->CurrentState == pkgCache::State::UnPacked ||
+	  pkg->CurrentState == pkgCache::State::HalfConfigured)
+    return pkg_unconfigured;
 
   return pkg_unchanged;
 }
@@ -802,8 +809,8 @@ static bool internal_is_interesting_dep(const pkgCache::DepIterator &d,
 	  while(!d2.end())
 	    {
 	      if(d2->Type == pkgCache::Dep::Recommends &&
-		 or_group_subsumes(d2, d, &cache->GetCache()) ||
-		 or_group_subsumes(d, d2, &cache->GetCache()))
+		 (or_group_subsumes(d2, d, &cache->GetCache()) ||
+		  or_group_subsumes(d, d2, &cache->GetCache())))
 		{
 		  pkgCache::DepIterator dtmp = d;
 		  while(!dtmp.end() && dtmp->CompareOp & pkgCache::Dep::Or)
@@ -868,7 +875,7 @@ std::wstring get_short_description(const pkgCache::VerIterator &ver)
   if(vf.end())
     return std::wstring();
   else
-    return transcode(apt_package_records->Lookup(vf).ShortDesc(), "UTF-8");
+    return transcode(apt_package_records->Lookup(vf).ShortDesc());
 #else
   pkgCache::DescIterator d = ver.TranslatedDescription();
 
@@ -880,12 +887,10 @@ std::wstring get_short_description(const pkgCache::VerIterator &ver)
   if(df.end())
     return std::wstring();
   else
-    // Since Packages files don't have a bundled description and it
-    // just feels icky for the data in Packages to depend on the
-    // encoding, I just force a sane encoding on it here.  Really,
-    // though, Packages files should have an encapsulated encoding
-    // somewhere.
-    return transcode(apt_package_records->Lookup(df).ShortDesc(), "UTF-8");
+    // apt "helpfully" transcodes the description for us, instead of
+    // providing direct access to it.  So I need to assume that the
+    // description is encoded in the current locale.
+    return transcode(apt_package_records->Lookup(df).ShortDesc());
 #endif
 }
 
@@ -900,7 +905,7 @@ std::wstring get_long_description(const pkgCache::VerIterator &ver)
   if(vf.end())
     return std::wstring();
   else
-    return transcode(apt_package_records->Lookup(vf).LongDesc(), "UTF-8");
+    return transcode(apt_package_records->Lookup(vf).LongDesc());
 #else
   pkgCache::DescIterator d = ver.TranslatedDescription();
 
@@ -912,6 +917,6 @@ std::wstring get_long_description(const pkgCache::VerIterator &ver)
   if(df.end())
     return std::wstring();
   else
-    return transcode(apt_package_records->Lookup(df).LongDesc(), "UTF-8");
+    return transcode(apt_package_records->Lookup(df).LongDesc());
 #endif
 }
