@@ -1,6 +1,6 @@
 // aptcache.cc
 //
-//  Copyright 1999-2006 Daniel Burrows
+//  Copyright 1999-2007 Daniel Burrows
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -45,6 +45,8 @@
 #include <unistd.h>
 #include <stdio.h>
 #include <errno.h>
+#include <sys/stat.h>
+
 #include <generic/util/eassert.h>
 
 #include <sigc++/adaptors/bind.h>
@@ -64,25 +66,20 @@ class aptitudeDepCache::apt_undoer:public undoable
 {
   PkgIterator pkg;
   int prev_mode;  // One of Delete,Keep,Install
-  int prev_flags; /* For Delete, tracks Purge mode; for Install, tracks 
-		   * Reinstall mode..
-		   */
-  bool prev_autoinstalled;
-  changed_reason prev_installreason, prev_removereason;
+  int prev_iflags, prev_flags;
+  changed_reason prev_removereason;
   pkgCache::State::PkgSelectedState prev_selection_state;
 
   string prev_forbidver;
 
   aptitudeDepCache *owner;
 public:
-  apt_undoer(PkgIterator _pkg, int _prev_mode, int _prev_flags,
-	     changed_reason _prev_installreason,
+  apt_undoer(PkgIterator _pkg, int _prev_mode, int _prev_flags, int _prev_iflags,
 	     changed_reason _prev_removereason,
 	     pkgCache::State::PkgSelectedState _prev_selection_state,
 	     string _prev_forbidver,
 	     aptitudeDepCache *_owner)
-    :pkg(_pkg), prev_mode(_prev_mode), prev_flags(_prev_flags),
-     prev_installreason(_prev_installreason),
+    :pkg(_pkg), prev_mode(_prev_mode), prev_iflags(_prev_iflags), prev_flags(_prev_flags),
      prev_removereason(_prev_removereason),
      prev_selection_state(_prev_selection_state),
      prev_forbidver(_prev_forbidver),
@@ -92,24 +89,27 @@ public:
 
   void undo()
   {
-    if(prev_flags&ReInstall)
-      owner->internal_mark_install(pkg, false, true, NULL, false);
+    aptitudeDepCache::action_group group(*owner);
+
+    owner->pre_package_state_changed();
+
+    if(prev_iflags&ReInstall)
+      owner->internal_mark_install(pkg, false, true);
     else switch(prev_mode)
       {
       case ModeDelete:
-	// the unused_delete parameter isn't that important..
-	owner->internal_mark_delete(pkg, prev_flags&Purge, prev_removereason==unused, NULL, false);
+	owner->internal_mark_delete(pkg, prev_iflags & Purge, prev_removereason == unused);
 	break;
       case ModeKeep:
-	owner->internal_mark_keep(pkg, prev_flags&AutoKept, prev_selection_state==pkgCache::State::Hold, NULL, false);
+	owner->internal_mark_keep(pkg, prev_iflags & AutoKept, prev_selection_state == pkgCache::State::Hold);
 	break;
       case ModeInstall:
-	owner->internal_mark_install(pkg, false, false, NULL, false);
+	owner->internal_mark_install(pkg, false, false);
 	break;
       }
 
     // make sure that everything is really set.
-    owner->get_ext_state(pkg).install_reason=prev_installreason;
+    owner->MarkAuto(pkg, (prev_flags & Flag::Auto));
     owner->get_ext_state(pkg).remove_reason=prev_removereason;
     owner->get_ext_state(pkg).forbidver=prev_forbidver;
   }
@@ -161,31 +161,49 @@ public:
 
   void undo()
   {
+    owner->pre_package_state_changed();
     owner->set_candidate_version(oldver, NULL);
+    owner->package_state_changed();
   }
 };
 
+aptitudeDepCache::action_group::action_group(aptitudeDepCache &cache, undo_group *group)
+  : parent_group(new pkgDepCache::ActionGroup(cache)),
+    cache(cache), group(group)
+{
+  cache.begin_action_group();
+}
+
+aptitudeDepCache::action_group::~action_group()
+{
+  // Force the parent to mark-and-sweep first.
+  delete parent_group;
+
+  cache.end_action_group(group);
+}
+
 aptitudeDepCache::aptitudeDepCache(pkgCache *Cache, Policy *Plcy)
   :pkgDepCache(Cache, Plcy), dirty(false), read_only(true),
-   package_states(NULL), lock(-1), group_level(0),
-   mark_and_sweep_in_progress(false)
+   package_states(NULL), lock(-1), group_level(0)
 {
   // When the "install recommended packages" flag changes, collect garbage.
+#if 0
   aptcfg->connect(PACKAGE "::Recommends-Important",
 		  sigc::bind(sigc::mem_fun(*this,
-					   &aptitudeDepCache::mark_and_sweep),
+					   &pkgDepCache::MarkAndSweep),
 			     (undo_group *) NULL));
 
   aptcfg->connect(PACKAGE "::Keep-Recommends",
 		  sigc::bind(sigc::mem_fun(*this,
-					   &aptitudeDepCache::mark_and_sweep),
+					   &pkgDepCache::MarkAndSweep),
 			     (undo_group *) NULL));
 
   // sim.
   aptcfg->connect(PACKAGE "::Keep-Suggests",
 		  sigc::bind(sigc::mem_fun(*this,
-					   &aptitudeDepCache::mark_and_sweep),
+					   &pkgDepCache::MarkAndSweep),
 			     (undo_group *) NULL));
+#endif
 }
 
 bool aptitudeDepCache::Init(OpProgress *Prog, bool WithLock, bool do_initselections, const char *status_fname)
@@ -210,12 +228,20 @@ bool aptitudeDepCache::build_selection_list(OpProgress &Prog, bool WithLock,
 					    bool do_initselections,
 					    const char *status_fname)
 {
+  action_group group(*this);
+
   bool initial_open=false;
   // This will be set to true if the state file does not exist.
 
   if(!pkgDepCache::Init(&Prog))
     return false;
 
+  // This is necessary so that the Garbage flags are initialized.
+  // Some of the Mark* methods perturb the Auto flag in largely
+  // uncontrollable ways, and one defense against this is to get
+  // Garbage set up (they behave more the way they ought to then).
+  MarkAndSweep();
+
   string statedir=aptcfg->FindDir("Dir::Aptitude::state", STATEDIR);
   // Should this not go under Dir:: ?  I'm not sure..
   delete package_states;
@@ -224,8 +250,8 @@ bool aptitudeDepCache::build_selection_list(OpProgress &Prog, bool WithLock,
     {
       package_states[i].new_package=true;
       package_states[i].reinstall=false;
-      package_states[i].install_reason=manual;
       package_states[i].remove_reason=manual;
+      package_states[i].selection_state = pkgCache::State::Unknown;
     }
 
   if(WithLock && lock==-1)
@@ -262,14 +288,20 @@ bool aptitudeDepCache::build_selection_list(OpProgress &Prog, bool WithLock,
       if(errno!=ENOENT)
 	_error->Warning(_("Can't open Aptitude extended state file"));
       else
-	initial_open=true;
+	{
+	  initial_open=true;
+	  // Mark the cache as dirty so that we'll create the
+	  // pkgstates file later.  We need to do this even if the
+	  // user doesn't change anything because otherwise we won't
+	  // know which packages are new until we save the cache (#429732).
+	  dirty = true;
+	}
     }
   else
     {
       int file_size=state_file.Size();
       Prog.OverallProgress(0, file_size, 1, _("Reading extended state information"));
 
-      begin_action_group();
       pkgTagFile tagfile(&state_file);
       pkgTagSection section;
       int amt=0;
@@ -297,10 +329,16 @@ bool aptitudeDepCache::build_selection_list(OpProgress &Prog, bool WithLock,
 	      // The install reason is much more important to preserve
 	      // from previous versions, so support the outdated name
 	      // for it.
-	      pkg_state.install_reason=(changed_reason)
+	      changed_reason install_reason=(changed_reason)
 		section.FindI("Install-Reason",
 			      section.FindI("Last-Change", manual));
 
+	      if(install_reason != manual)
+		{
+		  MarkAuto(pkg, true);
+		  dirty = true;
+		}
+
 	      pkg_state.remove_reason=(changed_reason)
 		section.FindI("Remove-Reason", manual);
 
@@ -313,18 +351,36 @@ bool aptitudeDepCache::build_selection_list(OpProgress &Prog, bool WithLock,
 	      pkg_state.candver=candver;
 	      pkg_state.forbidver=section.FindS("ForbidVer");
 
-	      if(do_dselect && pkg->SelectedState != last_dselect_state &&
-		 do_initselections)
+	      if(do_dselect && pkg->SelectedState != last_dselect_state)
 		{
 		  MarkFromDselect(pkg);
+		  // dirty should be set to "true" so that we update
+		  // the on-disk dselect state ASAP, even if no
+		  // package states change as a result.
 		  dirty=true;
+
+		  // We need to update the package state from the
+		  // dselect state regardless of whether we're doing
+		  // initselections.  This is so that, e.g., if the
+		  // user installed a package outside aptitude (so the
+		  // dselect state says to install it), our internal
+		  // state isn't left at "remove".  But if we aren't
+		  // supposed to set up stored installs/removals, we
+		  // should cancel this at the apt-get level (so the
+		  // package doesn't get changed if dselect said to
+		  // install it, but this isn't stored in our database
+		  // for future runs).
+		  //
+		  // In the past, we skipped doing MarkFromDselect in
+		  // this case.  BAD.
+		  if(!do_initselections)
+		    MarkKeep(pkg, false);
 		}
 	    }
 	  amt+=section.size();
 	  Prog.OverallProgress(amt, file_size, 1, _("Reading extended state information"));
 	}
       Prog.OverallProgress(file_size, file_size, 1, _("Reading extended state information"));
-      end_action_group(NULL);
     }
 
   int num=0;
@@ -415,16 +471,6 @@ bool aptitudeDepCache::build_selection_list(OpProgress &Prog, bool WithLock,
   if(aptcfg->FindB(PACKAGE "::Auto-Upgrade", false) && do_initselections)
     mark_all_upgradable(aptcfg->FindB(PACKAGE "::Auto-Install", true),
 			true, NULL);
-  else
-    {
-      // Normally this was done in the mark_all_upgradable, but we no
-      // longer do that by default.
-      mark_and_sweep(NULL);
-      cleanup_after_change(NULL);
-    }
-
-  duplicate_cache(&backup_state);
-
   Prog.Done();
 
   read_only = (lock == -1);
@@ -443,7 +489,7 @@ void aptitudeDepCache::mark_all_upgradable(bool with_autoinst,
       return;
     }
 
-  begin_action_group();
+  action_group group(*this, undo);
 
   for(int iter=0; iter==0 || (iter==1 && with_autoinst); ++iter)
     {
@@ -503,9 +549,6 @@ void aptitudeDepCache::mark_all_upgradable(bool with_autoinst,
 	    }
 	}
     }
-
-  // will handle setting undos, mark-and-sweep, etc:
-  end_action_group(undo);
 }
 
 // If fd is -1, just write unconditionally to the given fd.
@@ -523,14 +566,21 @@ bool aptitudeDepCache::save_selection_list(OpProgress &prog,
 
   if(lock==-1 && !status_fname)
     return true;
+
+  writeStateFile(&prog);
+
   string statefile=_config->FindDir("Dir::Aptitude::state", STATEDIR)+"pkgstates";
 
   FileFd newstate;
 
   if(!status_fname)
-    newstate.Open(statefile+".new", FileFd::WriteEmpty);
+    newstate.Open(statefile+".new", FileFd::WriteEmpty, 0644);
   else
-    newstate.Open(status_fname, FileFd::WriteEmpty);
+    newstate.Open(status_fname, FileFd::WriteEmpty, 0644);
+
+  // The user might have a restrictive umask -- make sure we get a
+  // mode 644 file.
+  fchmod(newstate.Fd(), 0644);
 
   if(!newstate.IsOpen())
     _error->Error(_("Cannot open Aptitude state file"));
@@ -563,12 +613,11 @@ bool aptitudeDepCache::save_selection_list(OpProgress &prog,
 
 	    len=snprintf(buf,
 			 400,
-			 "Package: %s\nUnseen: %s\nState: %i\nDselect-State: %i\nLast-Change: %i\nRemove-Reason: %i\n%s%s%s\n",
+			 "Package: %s\nUnseen: %s\nState: %i\nDselect-State: %i\nRemove-Reason: %i\n%s%s%s\n",
 			 i.Name(),
 			 estate.new_package?"yes":"no",
 			 estate.selection_state,
 			 i->SelectedState,
-			 estate.install_reason,
 			 estate.remove_reason,
 			 upgradestr.c_str(),
 			 forbidstr.c_str(),
@@ -704,8 +753,8 @@ void aptitudeDepCache::forget_new(undoable **undoer)
 
 undoable *aptitudeDepCache::state_restorer(PkgIterator pkg, StateCache &state, aptitude_state &ext_state)
 {
-  return new apt_undoer(pkg, state.Mode, state.iFlags,
-			ext_state.install_reason, ext_state.remove_reason,
+  return new apt_undoer(pkg, state.Mode, state.Flags, state.iFlags,
+			ext_state.remove_reason,
 			ext_state.selection_state,
 			ext_state.forbidver, this);
 }
@@ -715,12 +764,21 @@ void aptitudeDepCache::cleanup_after_change(undo_group *undo, bool alter_stickie
   // selected_state if it's not already updated; (b) adds an item to the
   // undo group.
 {
+  // We get here with NULL backup_state in certain very early failures
+  // (e.g., when someone else is holding a lock).  In this case we
+  // don't know what the previous state was, so we can't possibly
+  // build a collection of undoers to return to it.
+  if(backup_state.PkgState == NULL ||
+     backup_state.DepState == NULL ||
+     backup_state.AptitudeState == NULL)
+    return;
+
   for(pkgCache::PkgIterator pkg=PkgBegin(); !pkg.end(); pkg++)
     {
       if(PkgState[pkg->ID].Mode!=backup_state.PkgState[pkg->ID].Mode ||
+	 (PkgState[pkg->ID].Flags & pkgCache::Flag::Auto) != (backup_state.PkgState[pkg->ID].Flags & pkgCache::Flag::Auto) ||
 	 package_states[pkg->ID].selection_state!=backup_state.AptitudeState[pkg->ID].selection_state ||
 	 package_states[pkg->ID].reinstall!=backup_state.AptitudeState[pkg->ID].reinstall ||
-	 package_states[pkg->ID].install_reason!=backup_state.AptitudeState[pkg->ID].install_reason ||
 	 package_states[pkg->ID].remove_reason!=backup_state.AptitudeState[pkg->ID].remove_reason ||
 	 package_states[pkg->ID].forbidver!=backup_state.AptitudeState[pkg->ID].forbidver)
 	{
@@ -749,12 +807,6 @@ void aptitudeDepCache::cleanup_after_change(undo_group *undo, bool alter_stickie
 		    }
 		  break;
 		case ModeKeep:
-		  // If this was going to be deleted, assume it was
-		  // automatically marked to be installed.
-		  if(package_states[pkg->ID].selection_state==pkgCache::State::DeInstall ||
-		     package_states[pkg->ID].selection_state==pkgCache::State::Purge)
-		    package_states[pkg->ID].install_reason=libapt;
-
 		  if(!pkg.CurrentVer().end())
 		    package_states[pkg->ID].selection_state=pkgCache::State::Install;
 		  else if(pkg->CurrentState==pkgCache::State::NotInstalled)
@@ -764,15 +816,7 @@ void aptitudeDepCache::cleanup_after_change(undo_group *undo, bool alter_stickie
 		  break;
 		case ModeInstall:
 		  if(package_states[pkg->ID].selection_state!=pkgCache::State::Install)
-		    {
-		      package_states[pkg->ID].selection_state=pkgCache::State::Install;
-		      // Only set it to not be manual if it's a new
-		      // install, or the package was going to be
-		      // removed.
-		      if(pkg.CurrentVer().end() ||
-			 backup_state.PkgState[pkg->ID].Delete())
-			package_states[pkg->ID].install_reason=libapt;
-		    }
+		    package_states[pkg->ID].selection_state=pkgCache::State::Install;
 		  break;
 		}
 	    }
@@ -785,11 +829,10 @@ void aptitudeDepCache::cleanup_after_change(undo_group *undo, bool alter_stickie
     }
 }
 
-void aptitudeDepCache::internal_mark_install(const PkgIterator &Pkg,
-					     bool AutoInst,
-					     bool ReInstall,
-					     undo_group *undo,
-					     bool do_mark_and_sweep)
+void aptitudeDepCache::mark_install(const PkgIterator &Pkg,
+				    bool AutoInst,
+				    bool ReInstall,
+				    undo_group *undo)
 {
   if(read_only && !read_only_permission())
     {
@@ -798,17 +841,32 @@ void aptitudeDepCache::internal_mark_install(const PkgIterator &Pkg,
       return;
     }
 
+  action_group group(*this, undo);
+
   pre_package_state_changed();
+
+  internal_mark_install(Pkg, AutoInst, ReInstall);
+}
+
+void aptitudeDepCache::internal_mark_install(const PkgIterator &Pkg,
+					     bool AutoInst,
+					     bool ReInstall)
+{
   dirty=true;
 
-  // Only change the install reason if the package was not previously
-  // installed OR it was going to be removed because it was gc'ed.
-  if((Pkg.CurrentVer().end() && !(*this)[Pkg].Install()) ||
-     (!Pkg.CurrentVer().end() && get_ext_state(Pkg).garbage))
-    get_ext_state(Pkg).install_reason=manual;
+
+  bool set_to_manual =
+    ((Pkg.CurrentVer().end()  && !(*this)[Pkg].Install()) ||
+     (!Pkg.CurrentVer().end() && (*this)[Pkg].Delete() && get_ext_state(Pkg).remove_reason == unused));
+
+  if(set_to_manual)
+    MarkAuto(Pkg, false);
+
 
   if(!ReInstall)
-    pkgDepCache::MarkInstall(Pkg, AutoInst);
+    {
+      pkgDepCache::MarkInstall(Pkg, AutoInst);
+    }
   else
     pkgDepCache::MarkKeep(Pkg, AutoInst);
 
@@ -817,25 +875,12 @@ void aptitudeDepCache::internal_mark_install(const PkgIterator &Pkg,
   get_ext_state(Pkg).selection_state=pkgCache::State::Install;
   get_ext_state(Pkg).reinstall=ReInstall;
   get_ext_state(Pkg).forbidver="";
-
-  if(group_level==0)
-    {
-      if(do_mark_and_sweep)
-	mark_and_sweep(undo);
-
-      cleanup_after_change(undo);
-
-      duplicate_cache(&backup_state);
-
-      package_state_changed();
-    }
 }
 
-void aptitudeDepCache::internal_mark_delete(const PkgIterator &Pkg,
-					    bool Purge,
-					    bool unused_delete,
-					    undo_group *undo,
-					    bool do_mark_and_sweep)
+void aptitudeDepCache::mark_delete(const PkgIterator &Pkg,
+				   bool Purge,
+				   bool unused_delete,
+				   undo_group *undo)
 {
   if(read_only && !read_only_permission())
     {
@@ -844,7 +889,17 @@ void aptitudeDepCache::internal_mark_delete(const PkgIterator &Pkg,
       return;
     }
 
+  action_group group(*this, undo);
+
   pre_package_state_changed();
+
+  internal_mark_delete(Pkg, Purge, unused_delete);
+}
+
+void aptitudeDepCache::internal_mark_delete(const PkgIterator &Pkg,
+					    bool Purge,
+					    bool unused_delete)
+{
   dirty=true;
 
   bool previously_to_delete=(*this)[Pkg].Delete();
@@ -862,23 +917,9 @@ void aptitudeDepCache::internal_mark_delete(const PkgIterator &Pkg,
       else
 	get_ext_state(Pkg).remove_reason=manual;
     }
-
-  // Argh argh argh.  Doing any of this when there's a mark-and-sweep
-  // running causes extreme screwiness.
-  if(!mark_and_sweep_in_progress && group_level==0)
-    {
-      if(do_mark_and_sweep)
-	mark_and_sweep(undo);
-
-      cleanup_after_change(undo);
-
-      duplicate_cache(&backup_state);
-
-      package_state_changed();
-    }
 }
 
-void aptitudeDepCache::internal_mark_keep(const PkgIterator &Pkg, bool Soft, bool SetHold, undo_group *undo, bool do_mark_and_sweep)
+void aptitudeDepCache::mark_keep(const PkgIterator &Pkg, bool Automatic, bool SetHold, undo_group *undo)
 {
   if(read_only && !read_only_permission())
     {
@@ -887,17 +928,32 @@ void aptitudeDepCache::internal_mark_keep(const PkgIterator &Pkg, bool Soft, boo
       return;
     }
 
+  action_group group(*this, undo);
+
   pre_package_state_changed();
+
+  internal_mark_keep(Pkg, Automatic, SetHold);
+}
+
+void aptitudeDepCache::internal_mark_keep(const PkgIterator &Pkg, bool Automatic, bool SetHold)
+{
   dirty=true;
 
-  // On an unused package, this has the effect of making it manually
-  // installed.  (check the current version to avoid adjusting
-  // already-removed packages that are being purged)
-  if((*this)[Pkg].Delete() && !Pkg.CurrentVer().end() &&
-     get_ext_state(Pkg).remove_reason==unused)
-    get_ext_state(Pkg).install_reason=manual;
 
-  pkgDepCache::MarkKeep(Pkg, Soft);
+  // If the package is currently installed and is being garbage
+  // collected, switch it to manual mode.  We check
+  // pkg.CurrentVer().end() to avoid fiddling with packages that are
+  // in the ConfigFiles state and are being purged.
+  bool was_garbage_removed =
+    (*this)[Pkg].Delete() &&
+    !Pkg.CurrentVer().end() &&
+    get_ext_state(Pkg).remove_reason == unused;
+
+  if(was_garbage_removed)
+    MarkAuto(Pkg, false);
+
+
+  pkgDepCache::MarkKeep(Pkg, false, !Automatic);
   pkgDepCache::SetReInstall(Pkg, false);
   get_ext_state(Pkg).reinstall=false;
 
@@ -912,18 +968,6 @@ void aptitudeDepCache::internal_mark_keep(const PkgIterator &Pkg, bool Soft, boo
     get_ext_state(Pkg).selection_state=pkgCache::State::Hold;
   else
     get_ext_state(Pkg).selection_state=pkgCache::State::Install;
-
-  if(group_level==0)
-    {
-      if(do_mark_and_sweep)
-	mark_and_sweep(undo);
-
-      cleanup_after_change(undo);
-
-      duplicate_cache(&backup_state);
-
-      package_state_changed();
-    }
 }
 
 void aptitudeDepCache::set_candidate_version(const VerIterator &ver,
@@ -944,6 +988,21 @@ void aptitudeDepCache::set_candidate_version(const VerIterator &ver,
        ver.ParentPkg()->CurrentState != pkgCache::State::ConfigFiles)))
     {
       pre_package_state_changed();
+
+
+      // Make the package manually installed if it was being
+      // garbage-collected.
+      bool set_to_manual =
+	(ver.ParentPkg().CurrentVer().end() && !(*this)[ver.ParentPkg()].Install()) ||
+	(!ver.ParentPkg().CurrentVer().end() &&
+	 (*this)[ver.ParentPkg()].Delete() &&
+	 get_ext_state(ver.ParentPkg()).remove_reason == unused);
+
+      if(set_to_manual)
+	MarkAuto(ver.ParentPkg(), false);
+
+
+
       // Use the InstVerIter instead of GetCandidateVersion, since
       // that seems to store the currently to-be-installed version.
       VerIterator prev=(*this)[(ver.ParentPkg())].InstVerIter(GetCache());
@@ -964,7 +1023,7 @@ void aptitudeDepCache::set_candidate_version(const VerIterator &ver,
 	  if(undo)
 	    undo->add_item(new candver_undoer(prev, this));
 
-	  mark_and_sweep(undo);
+	  MarkAndSweep();
 
 	  //if(BrokenCount()>0)
 	  //create_resolver();
@@ -991,6 +1050,8 @@ void aptitudeDepCache::forbid_upgrade(const PkgIterator &pkg,
 
   if(verstr!=estate.forbidver)
     {
+      action_group group(*this, undo);
+
       pre_package_state_changed();
 
       pkgCache::VerIterator candver=(*this)[pkg].CandidateVerIter(*this);
@@ -1000,17 +1061,6 @@ void aptitudeDepCache::forbid_upgrade(const PkgIterator &pkg,
       estate.forbidver=verstr;
       if(!candver.end() && candver.VerStr()==verstr && (*this)[pkg].Install())
 	MarkKeep(pkg, false);
-
-      if(group_level==0)
-	{
-	  mark_and_sweep(undo);
-
-	  cleanup_after_change(undo, false);
-
-	  duplicate_cache(&backup_state);
-
-	  package_state_changed();
-	}
     }
 }
 
@@ -1023,24 +1073,23 @@ void aptitudeDepCache::mark_single_install(const PkgIterator &Pkg, undo_group *u
       return;
     }
 
+  action_group group(*this, undo);
+
   pre_package_state_changed();
   dirty=true;
 
   for(PkgIterator i=PkgBegin(); !i.end(); i++)
     pkgDepCache::MarkKeep(i, true);
 
+
+  bool set_to_manual =
+    ((Pkg.CurrentVer().end()  && !(*this)[Pkg].Install()) ||
+     (!Pkg.CurrentVer().end() && (*this)[Pkg].Delete() && get_ext_state(Pkg).remove_reason == unused));
+
+  if(set_to_manual)
+    MarkAuto(Pkg, false);
+
   pkgDepCache::MarkInstall(Pkg, true);
-
-  if(group_level==0)
-    {
-      mark_and_sweep(undo);
-
-      cleanup_after_change(undo, false);
-
-      duplicate_cache(&backup_state);
-
-      package_state_changed();
-    }
 }
 
 void aptitudeDepCache::mark_auto_installed(const PkgIterator &Pkg,
@@ -1054,21 +1103,12 @@ void aptitudeDepCache::mark_auto_installed(const PkgIterator &Pkg,
       return;
     }
 
+  action_group group(*this, undo);
+
   pre_package_state_changed();
   dirty=true;
 
-  get_ext_state(Pkg).install_reason=set_auto?user_auto:manual;
-
-  if(group_level==0)
-    {
-      mark_and_sweep(undo);
-
-      cleanup_after_change(undo);
-
-      duplicate_cache(&backup_state);
-
-      package_state_changed();
-    }
+  MarkAuto(Pkg, set_auto);
 }
 
 bool aptitudeDepCache::all_upgrade(bool with_autoinst, undo_group *undo)
@@ -1080,6 +1120,8 @@ bool aptitudeDepCache::all_upgrade(bool with_autoinst, undo_group *undo)
       return false;
     }
 
+  action_group group(*this, undo);
+
   pre_package_state_changed();
 
   pkgProblemResolver fixer(this);
@@ -1099,17 +1141,6 @@ bool aptitudeDepCache::all_upgrade(bool with_autoinst, undo_group *undo)
 
   bool rval=fixer.ResolveByKeep();
 
-  if(group_level==0)
-    {
-      mark_and_sweep(undo);
-
-      cleanup_after_change(undo);
-
-      duplicate_cache(&backup_state);
-
-      package_state_changed();
-    }
-
   return rval;
 }
 
@@ -1122,6 +1153,8 @@ bool aptitudeDepCache::try_fix_broken(pkgProblemResolver &fixer, undo_group *und
       return false;
     }
 
+  action_group group(*this, undo);
+
   pre_package_state_changed();
   dirty=true;
   bool founderr=false;
@@ -1131,17 +1164,6 @@ bool aptitudeDepCache::try_fix_broken(pkgProblemResolver &fixer, undo_group *und
   if(founderr)
     _error->Error(_("Unable to correct dependencies, some packages cannot be installed"));
 
-  if(group_level==0)
-    {
-      mark_and_sweep(undo);
-
-      cleanup_after_change(undo);
-
-      duplicate_cache(&backup_state);
-
-      package_state_changed();
-    }
-
   return !founderr;
 }
 
@@ -1250,6 +1272,59 @@ void aptitudeDepCache::duplicate_cache(apt_state_snapshot *target)
   target->iBadCount=iBadCount;
 }
 
+void aptitudeDepCache::sweep()
+{
+  if(!aptcfg->FindB(PACKAGE "::Delete-Unused", true))
+    return;
+
+  // Suppress intermediate removals.
+  //
+  // \todo this may cause problems if we do undo tracking via ActionGroups.
+  pkgDepCache::ActionGroup group(*this);
+
+  bool purge_unused = aptcfg->FindB(PACKAGE "::Purge-Unused", false);
+
+  for(pkgCache::PkgIterator pkg = PkgBegin(); !pkg.end(); ++pkg)
+    {
+      if(PkgState[pkg->ID].Garbage)
+	{
+	  if(pkg.CurrentVer() != 0 && pkg->CurrentState != pkgCache::State::ConfigFiles)
+	    {
+	      // NB: apt sets the Garbage flag on packages that are
+	      // being deleted and are automatic (this is due to some
+	      // internal details of the mark&sweep algorithm and its
+	      // aptitude heritage).  To compensate for this, we
+	      // *only* set the unused-delete flag if the package was
+	      // not previously being deleted.
+	      if(!PkgState[pkg->ID].Delete())
+		{
+		  MarkDelete(pkg, purge_unused);
+		  package_states[pkg->ID].selection_state =
+		    (purge_unused ? pkgCache::State::Purge : pkgCache::State::DeInstall);
+		  package_states[pkg->ID].remove_reason = unused;
+		}
+	    }
+	  else
+	    {
+	      if(pkg.CurrentVer().end())
+		{
+		  if((*this)[pkg].iFlags & Purge)
+		    package_states[pkg->ID].selection_state = pkgCache::State::Purge;
+		  else
+		    package_states[pkg->ID].selection_state = pkgCache::State::DeInstall;
+		}
+	      else
+		package_states[pkg->ID].selection_state = pkgCache::State::Install;
+	      MarkKeep(pkg, false, false);
+	    }
+	}
+      else if(PkgState[pkg->ID].Delete() && package_states[pkg->ID].remove_reason == unused)
+	{
+	  MarkKeep(pkg, false, false);
+	}
+    }
+}
+
 void aptitudeDepCache::begin_action_group()
 {
   group_level++;
@@ -1265,10 +1340,12 @@ void aptitudeDepCache::end_action_group(undo_group *undo)
 	{
 	  if(group_level == 0)
 	    read_only_fail();
+
+	  group_level--;
 	  return;
 	}
 
-      mark_and_sweep(undo);
+      sweep();
 
       cleanup_after_change(undo);
 
@@ -1314,226 +1391,6 @@ void aptitudeDepCache::restore_apt_state(const apt_state_snapshot *snapshot)
   iBadCount=snapshot->iBadCount;
 }
 
-// Mark-and-sweep starts here.
-//
-// If a package version is not installed and is not going to be installed, it
-// is not visited further.  Otherwise, the appropriate mark is set.
-//
-//  FIXME: follow Recommends iff the appropriate stuff is set.
-void aptitudeDepCache::mark_package(const PkgIterator &pkg,
-				    const VerIterator &ver,
-				    bool follow_recommends,
-				    bool follow_suggests,
-				    bool debug,
-				    int level)
-{
-  pkgDepCache::StateCache &state=(*this)[pkg];
-  aptitude_state &estate=get_ext_state(pkg);
-  pkgCache::VerIterator candver=state.CandidateVerIter(*this);
-  pkgCache::VerIterator instver=state.InstVerIter(*this);
-
-  // If a package was garbage-collected but is now being marked, we
-  // should re-select it.  (note: undo==NULL is actually correct;
-  // mark_and_sweep *never* stores undo info....)
-  if(state.Delete() && estate.remove_reason==unused)
-    {
-      if(ver==candver)
-	{
-	  if(debug)
-	    std::cout << string(level*2, ' ')
-		      << "Cancelling the deletion of "
-		      << pkg.Name() << " and installing its candidate version ("
-		      << ver.VerStr() << ")" << std::endl;
-
-	  mark_install(pkg, false, false, NULL);
-	}
-      else if(ver==pkg.CurrentVer())
-	{
-	  if(debug)
-	    std::cout << string(level*2, ' ')
-		      << "Cancelling the deletion of "
-		      << pkg.Name() << " and reverting to its currently installed version ("
-		      << ver.VerStr() << ")" << std::endl;
-
-	  pre_package_state_changed();
-	  MarkKeep(pkg);
-	}
-
-      instver=state.InstVerIter(*this);
-    }
-
-  if(!(ver==instver && !instver.end()))
-    {
-      if(debug)
-	{
-	  std::cout << string(level*2, ' ')
-		    << "Not marking " << pkg.Name()
-		    << " " << (ver.end() ? "[UNINST]" : ver.VerStr())
-		    << ": ";
-
-	  if(ver == instver)
-	    std::cout << "it is the UNINST version";
-	  else
-	    std::cout << "it is not the version to be installed ("
-		      << (instver.end() ? "[UNINST]" : instver.VerStr())
-		      << ")" << std::endl;
-	}
-      return;
-    }
-
-  if(estate.marked)
-    {
-      if(debug)
-	std::cout << string(level*2, ' ')
-		  << "Not marking " << pkg.Name()
-		  << " " << (ver.end() ? "[UNINST]" : ver.VerStr())
-		  << ": it is already marked." << std::endl;
-      return;
-    }
-
-  if(debug)
-    std::cout << string(level*2, ' ')
-	      << "Marking " << pkg.Name() << " "
-	      << (ver.end() ? "[UNINST]" : ver.VerStr())
-	      << std::endl;
-  estate.marked=true;
-
-  if(!ver.end())
-    {
-      for(DepIterator d=ver.DependsList(); !d.end(); ++d)
-	{
-	  if(d->Type==pkgCache::Dep::Depends ||
-	     d->Type==pkgCache::Dep::PreDepends ||
-	     (follow_recommends &&
-	      d->Type==pkgCache::Dep::Recommends) ||
-	     (follow_suggests &&
-	      d->Type==pkgCache::Dep::Suggests))
-	    {
-	      if(debug)
-		{
-		  std::cout << string(level*2, ' ')
-			    << "Following dependency "
-			    << d.ParentPkg().Name()
-			    << " "
-			    << d.DepType()
-			    << " "
-			    << d.TargetPkg().Name();
-		  if(d.TargetVer() != NULL)
-		    std::cout << " ("
-			      << d.CompType()
-			      << " "
-			      << d.TargetVer()
-			      << ")";
-		  std::cout << std::endl;
-		}
-
-	      // Try all versions of this package.
-	      for(VerIterator V=d.TargetPkg().VersionList(); !V.end(); ++V)
-		if(_system->VS->CheckDep(V.VerStr(), d->CompareOp, d.TargetVer()))
-		  mark_package(V.ParentPkg(), V,
-			       follow_recommends, follow_suggests, debug, level + 1);
-
-	      // Now try virtual packages
-	      for(PrvIterator prv=d.TargetPkg().ProvidesList(); !prv.end(); ++prv)
-		if(_system->VS->CheckDep(prv.ProvideVersion(), d->CompareOp, d.TargetVer()))
-		  mark_package(prv.OwnerPkg(), prv.OwnerVer(),
-			       follow_recommends, follow_suggests, debug, level + 1);
-	    }
-	}
-    }
-}
-
-// The root-set is every essential package, as well as those packages which
-// have been manually chosen for installation, and are planned to be installed.
-void aptitudeDepCache::mark_and_sweep(undo_group *undo)
-{
-  // EWW.
-  if(mark_and_sweep_in_progress)
-    return;
-
-  begin_action_group();
-
-  mark_and_sweep_in_progress=true;
-
-  std::string matchterm = aptcfg->Find(PACKAGE "::Keep-Unused-Pattern", "~nlinux-image-.*");
-  if(matchterm.empty()) // Bug-compatibility with old versions.
-    matchterm = aptcfg->Find(PACKAGE "::Delete-Unused-Pattern", "");
-  pkg_matcher *matcher=matchterm.empty()?NULL:parse_pattern(matchterm);
-
-  for(pkgCache::PkgIterator p=PkgBegin(); !p.end(); ++p)
-    {
-      package_states[p->ID].marked=false;
-      package_states[p->ID].garbage=false;
-    }
-
-  bool follow_recommends=aptcfg->FindB(PACKAGE "::Recommends-Important", true) ||
-    aptcfg->FindB(PACKAGE "::Keep-Recommends", false);
-  bool follow_suggests=aptcfg->FindB(PACKAGE "::Keep-Suggests", false) ||
-    aptcfg->FindB(PACKAGE "::Suggests-Important", false);
-  bool debug = aptcfg->FindB(PACKAGE "::GC-Debug", false);
-
-  if(debug)
-    std::cout << "*** BEGIN MARK AND SWEEP ***" << std::endl;
-
-  for(pkgCache::PkgIterator p=PkgBegin(); !p.end(); ++p)
-    {
-      if(package_states[p->ID].install_reason==manual ||
-	 (p->Flags & pkgCache::Flag::Essential) ||
-	 (matcher && ((PkgState[p->ID].Keep() &&
-		       !p.CurrentVer().end() &&
-		       matcher->matches(p)) ||
-		      (PkgState[p->ID].Install() &&
-		       matcher->matches(p)))))
-	{
-	  if(PkgState[p->ID].Keep() && !p.CurrentVer().end())
-	    mark_package(p, p.CurrentVer(),
-			 follow_recommends, follow_suggests, debug, 0);
-	  else if(PkgState[p->ID].Install())
-	    mark_package(p, PkgState[p->ID].InstVerIter(GetCache()),
-			 follow_recommends, follow_suggests, debug, 0);
-	}
-    }
-
-  bool delete_unused=aptcfg->FindB(PACKAGE "::Delete-Unused", true);
-
-  for(pkgCache::PkgIterator p=PkgBegin(); !p.end(); ++p)
-    {
-      StateCache &state=(*this)[p];
-      aptitude_state &estate=get_ext_state(p);
-
-      if(!estate.marked)
-	{
-	  estate.garbage=true;
-
-	  // Be sure not to re-delete already deleted packages.
-	  if(delete_unused && (!p.CurrentVer().end() || state.Install()) &&
-	     !state.Delete())
-	    {
-	      bool do_delete=true;
-
-	      if(debug)
-		std::cout << "*** Removing unused package " << p.Name() << std::endl;
-
-	      if(do_delete)
-		mark_delete(p,
-			    aptcfg->FindB(PACKAGE "::Purge-Unused", false),
-			    true,
-			    undo);
-	    }
-	}
-    }
-
-  mark_and_sweep_in_progress=false;
-
-  delete matcher;
-
-  if(debug)
-    std::cout << "*** END MARK AND SWEEP ***" << std::endl;
-
-  // end_action_group is written carefully so it works here:
-  end_action_group(undo);
-}
-
 void aptitudeDepCache::apply_solution(const generic_solution<aptitude_universe> &sol,
 				      undo_group *undo)
 {
@@ -1544,7 +1401,9 @@ void aptitudeDepCache::apply_solution(const generic_solution<aptitude_universe>
       return;
     }
 
-  begin_action_group();
+  action_group group(*this, undo);
+
+  pre_package_state_changed();
 
   for(imm::map<aptitude_resolver_package, generic_solution<aptitude_universe>::action>::const_iterator
 	i = sol.get_actions().begin();
@@ -1558,24 +1417,20 @@ void aptitudeDepCache::apply_solution(const generic_solution<aptitude_universe>
       if(actionver.end())
 	{
 	  // removal.
-	  internal_mark_delete(pkg, false, false, undo, false);
+	  internal_mark_delete(pkg, false, false);
 	  if(!curver.end())
-	    get_ext_state(pkg).remove_reason=from_resolver;
+	    get_ext_state(pkg).remove_reason = from_resolver;
 	}
       else if(actionver == curver)
-	// keep
-	internal_mark_keep(pkg, false, false, undo, false);
+	internal_mark_keep(pkg, true, false);
       else
 	// install a particular version that's not the current one.
 	{
-	  set_candidate_version(actionver, undo);
-	  internal_mark_install(pkg, false, false, undo, false);
-	  if(curver.end())
-	    get_ext_state(pkg).install_reason=from_resolver;
+	  set_candidate_version(actionver, NULL);
+	  internal_mark_install(pkg, false, false);
+	  MarkAuto(pkg, true);
 	}
     }
-
-  end_action_group(undo);  
 }
 
 aptitudeCacheFile::aptitudeCacheFile()
@@ -1675,3 +1530,85 @@ bool aptitudeDepCache::is_held(const PkgIterator &pkg)
     (state.selection_state == pkgCache::State::Hold ||
      (!candver.end() && candver.VerStr() == state.forbidver));
 }
+
+bool aptitudeDepCache::MarkFollowsRecommends()
+{
+  return pkgDepCache::MarkFollowsRecommends() ||
+    aptcfg->FindB(PACKAGE "::Recommends-Important", true) ||
+    aptcfg->FindB(PACKAGE "::Keep-Recommends", false);
+}
+
+bool aptitudeDepCache::MarkFollowsSuggests()
+{
+  return pkgDepCache::MarkFollowsSuggests() ||
+    aptcfg->FindB(PACKAGE "::Keep-Suggests", false) ||
+    aptcfg->FindB(PACKAGE "::Suggests-Important", false);
+}
+
+class AptitudeInRootSetFunc : public pkgDepCache::InRootSetFunc
+{
+  /** A matcher if one could be created; otherwise NULL. */
+  pkg_matcher *m;
+
+  /** \b true if the package was successfully constructed. */
+  bool constructedSuccessfully;
+
+  /** The secondary test to apply.  Only set if creating the
+   *  matcher succeeds.
+   */
+  pkgDepCache::InRootSetFunc *chain;
+public:
+  AptitudeInRootSetFunc(pkgDepCache::InRootSetFunc *_chain)
+    : m(NULL), constructedSuccessfully(false), chain(NULL)
+  {
+    std::string matchterm = aptcfg->Find(PACKAGE "::Keep-Unused-Pattern", "~nlinux-image-.*");
+    if(matchterm.empty()) // Bug-compatibility with old versions.
+      matchterm = aptcfg->Find(PACKAGE "::Delete-Unused-Pattern");
+
+    if(matchterm.empty())
+      constructedSuccessfully = true;
+    else
+      {
+	m = parse_pattern(matchterm);
+	if(m != NULL)
+	  constructedSuccessfully = true;
+      }
+
+    if(constructedSuccessfully)
+      chain = _chain;
+  }
+
+  bool wasConstructedSuccessfully() const
+  {
+    return constructedSuccessfully;
+  }
+
+  bool InRootSet(const pkgCache::PkgIterator &pkg)
+  {
+    if(m != NULL && m->matches(pkg))
+      return true;
+    else
+      return chain != NULL && chain->InRootSet(pkg);
+  }
+
+  ~AptitudeInRootSetFunc()
+  {
+    delete m;
+    delete chain;
+  }
+};
+
+pkgDepCache::InRootSetFunc *aptitudeDepCache::GetRootSetFunc()
+{
+  InRootSetFunc *superFunc = pkgDepCache::GetRootSetFunc();
+
+  AptitudeInRootSetFunc *f = new AptitudeInRootSetFunc(superFunc);
+
+  if(f->wasConstructedSuccessfully())
+    return f;
+  else
+    {
+      delete f;
+      return superFunc;
+    }
+}
