@@ -1,6 +1,6 @@
 // aptitude_resolver_universe.h                     -*-c++-*-
 //
-//   Copyright (C) 2005 Daniel Burrows
+//   Copyright (C) 2005, 2007 Daniel Burrows
 //
 //   This program is free software; you can redistribute it and/or
 //   modify it under the terms of the GNU General Public License as
@@ -429,7 +429,7 @@ class aptitude_resolver_package::version_iterator
     while(!ver.end() &&
 	  !ver.Downloadable() &&
 	  (ver != pkg.CurrentVer() || pkg->CurrentState == pkgCache::State::ConfigFiles))
-    ++ver;
+      ++ver;
   }
 public:
   /** \brief Create an invalid version_iterator. */
@@ -444,6 +444,7 @@ public:
 		   pkgDepCache *_cache)
     :cache(_cache), pkg(_pkg), ver(_pkg.VersionList())
   {
+    normalize();
   }
 
   /** \return The APT package corresponding to this abstract package. */
@@ -617,6 +618,25 @@ class aptitude_resolver_version::dep_iterator
    */
   bool prv_open;
 
+  /** \brief Walk forward on the full dependency graph (including
+   *  things that we filter out at the high level, like self-depends)
+   */
+  void advance();
+
+  static bool applicable(const pkgCache::DepIterator &dep,
+			 const pkgCache::PrvIterator &prv,
+			 bool prv_open,
+			 pkgDepCache *cache);
+
+  /** \return \b true if this dependency is one that should be made
+   *  visible in the dep graph.
+   *
+   *  Specifically, the dependency should not be a (direct or
+   *  indirect) self-depends, a non-interesting dependency, or
+   *  a conflict that can't be triggered.
+   */
+  bool applicable();
+
   void normalize();
 
 public:
@@ -833,11 +853,16 @@ bool aptitude_resolver_dep::broken_under(const InstallationType &I) const
       while(!dep.end())
 	{
 	  pkgCache::VerIterator direct_ver=I.version_of(aptitude_resolver_package(dep.TargetPkg(), cache)).get_ver();
-	  if(!direct_ver.end() &&
-	     _system->VS->CheckDep(direct_ver.VerStr(),
-				   dep->CompareOp,
-				   dep.TargetVer()))
-	    return false;
+	  if(!direct_ver.end())
+	    {
+	      const char * const direct_verstr = direct_ver.VerStr();
+	      const char * const dep_targetstr = dep.TargetVer();
+	      if(!direct_ver.end() &&
+		 _system->VS->CheckDep(direct_verstr,
+				       dep->CompareOp,
+				       dep_targetstr))
+		return false;
+	    }
 
 	  if(!dep.TargetVer())
 	    {
@@ -1145,4 +1170,7 @@ public:
 /** \brief Write an aptitude_resolver_dep to the given stream. */
 std::ostream &operator<<(ostream &out, aptitude_resolver_dep d);
 
+/** \brief Write an aptitude_resolver_version to the given stream. */
+std::ostream &operator<<(ostream &out, const aptitude_resolver_version &d);
+
 #endif
