@@ -2,7 +2,7 @@
 //
 //  The pkg_columnizer class.
 //
-//  Copyright 1999-2005 Daniel Burrows
+//  Copyright 1999-2005, 2007 Daniel Burrows
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -109,6 +109,14 @@ const char *pkg_item::pkg_columnizer::column_names[pkg_columnizer::numtypes]=
   };
 
 column_disposition pkg_item::pkg_columnizer::setup_column(int type)
+{
+  return setup_column(pkg, visible_ver, basex, type);
+}
+
+column_disposition pkg_item::pkg_columnizer::setup_column(const pkgCache::PkgIterator &pkg,
+							  const pkgCache::VerIterator &visible_ver,
+							  int basex,
+							  int type)
 {
   switch(type)
     {
@@ -323,9 +331,9 @@ column_disposition pkg_item::pkg_columnizer::setup_column(int type)
       break;
     case priority:
       if(!visible_ver.end() &&
-	 visible_ver.PriorityType() &&
-	 visible_ver.PriorityType()[0])
-	return column_disposition(visible_ver.PriorityType(), 0);
+	 const_cast<pkgCache::VerIterator &>(visible_ver).PriorityType() &&
+	 const_cast<pkgCache::VerIterator &>(visible_ver).PriorityType()[0])
+	return column_disposition(const_cast<pkgCache::VerIterator &>(visible_ver).PriorityType(), 0);
       else
 	return column_disposition(_("Unknown"), 0);
 
@@ -441,7 +449,7 @@ column_disposition pkg_item::pkg_columnizer::setup_column(int type)
       // or purged already and is not presently being installed.
       if((!pkg.CurrentVer().end() ||
 	  (*apt_cache_file)[pkg].Install()) &&
-	 (*apt_cache_file)->get_ext_state(pkg).install_reason!=aptitudeDepCache::manual)
+	 ((*apt_cache_file)[pkg].Flags & pkgCache::Flag::Auto))
 	return column_disposition("A", 0);
       else
 	return column_disposition("", 0);
@@ -485,17 +493,20 @@ column_disposition pkg_item::pkg_columnizer::setup_column(int type)
       {
 	int count=0;
 	char buf[100];
-	for(pkgCache::DepIterator D=pkg.RevDependsList(); !D.end(); D++)
-	  if(D.IsCritical() &&
-	     D->Type!=pkgCache::Dep::Conflicts &&
-	     D.ParentVer()==D.ParentPkg().CurrentVer() &&
-	     // That test is CORRECT; we want to see if the version
-	     // providing the dependency is correct.
-	     // (I'm writing this note because I just looked at the test
-	     //  and couldn't remember what it did despite writing it
-	     //  5 minutes ago. Maybe I should have my head examined :) )
-	     _system->VS->CheckDep(visible_ver.VerStr(), D->CompareOp, D.TargetVer()))
-	    count++;
+	if(!visible_ver.end())
+	  {
+	    for(pkgCache::DepIterator D=pkg.RevDependsList(); !D.end(); D++)
+	      if(D.IsCritical() &&
+		 D->Type!=pkgCache::Dep::Conflicts &&
+		 D.ParentVer()==D.ParentPkg().CurrentVer() &&
+		 // That test is CORRECT; we want to see if the version
+		 // providing the dependency is correct.
+		 // (I'm writing this note because I just looked at the test
+		 //  and couldn't remember what it did despite writing it
+		 //  5 minutes ago. Maybe I should have my head examined :) )
+		 _system->VS->CheckDep(visible_ver.VerStr(), D->CompareOp, D.TargetVer()))
+		count++;
+	  }
 
 	for(pkgCache::PrvIterator i=pkg.ProvidesList(); !i.end(); i++)
 	  for(pkgCache::DepIterator D=i.ParentPkg().RevDependsList(); !D.end(); D++)
