@@ -1,6 +1,6 @@
 // pkg_grouppolicy.cc
 //
-//  Copyright 1999-2005 Daniel Burrows
+//  Copyright 1999-2005, 2007 Daniel Burrows
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -368,11 +368,7 @@ void pkg_grouppolicy_status::add_package(const pkgCache::PkgIterator &pkg,
     {
       if(pkg_obsolete(pkg))
 	section=obsolete_pkg;
-      else if((*apt_cache_file)->get_ext_state(pkg).new_package)
-	section=newpkg;
-      else if(state.Status==2)
-	section=not_installed;
-      else if(state.Upgradable())
+      else if(state.Status != 2 && state.Upgradable())
 	{
 	  pkgCache::VerIterator candver=state.CandidateVerIter(*apt_cache_file);
 	  if(version_is_security(candver))
@@ -380,6 +376,10 @@ void pkg_grouppolicy_status::add_package(const pkgCache::PkgIterator &pkg,
 	  else
 	    section=upgradable;
 	}
+      else if((*apt_cache_file)->get_ext_state(pkg).new_package)
+	section=newpkg;
+      else if(state.Status==2)
+	section=not_installed;
       else
 	section=installed;
     }
@@ -541,6 +541,7 @@ const char * const pkg_grouppolicy_mode::child_names[num_pkg_action_states]=
   N_("Packages to be installed\n These packages have been manually selected for installation on your computer."),
   N_("Packages to be removed\n These packages have been manually selected for removal."),
   N_("Packages to be upgraded\n These packages will be upgraded to a newer version."),
+  N_("Packages that are partially installed\n These packages are not fully installed and configured; an attempt will be made to complete their installation."),
 };
 
 pkg_grouppolicy *pkg_grouppolicy_mode_factory::instantiate(pkg_signal *_sig,
@@ -895,13 +896,13 @@ void pkg_grouppolicy_task::init_section_descriptions()
   if(already_done)
     return;
 
-  task_section_descriptions.push_back(pair<string,wstring>("user", transcode(dgettext("tasksel", "End-user"))));
-  task_section_descriptions.push_back(pair<string,wstring>("server", transcode(dgettext("tasksel", "Servers"))));
-  task_section_descriptions.push_back(pair<string,wstring>("devel", transcode(dgettext("tasksel", "Development"))));
-  task_section_descriptions.push_back(pair<string,wstring>("l10n", transcode(dgettext("tasksel", "Localization"))));
-  task_section_descriptions.push_back(pair<string,wstring>("hware", transcode(dgettext("tasksel", "Hardware Support"))));
-  task_section_descriptions.push_back(pair<string,wstring>("misc", transcode(dgettext("tasksel", "Miscellaneous"))));
-  task_section_descriptions.push_back(pair<string,wstring>("unknown", transcode(dgettext("tasksel", "Unrecognized tasks"))));
+  task_section_descriptions.push_back(pair<string,wstring>("user", transcode(_("End-user"))));
+  task_section_descriptions.push_back(pair<string,wstring>("server", transcode(_("Servers"))));
+  task_section_descriptions.push_back(pair<string,wstring>("devel", transcode(_("Development"))));
+  task_section_descriptions.push_back(pair<string,wstring>("l10n", transcode(_("Localization"))));
+  task_section_descriptions.push_back(pair<string,wstring>("hware", transcode(_("Hardware Support"))));
+  task_section_descriptions.push_back(pair<string,wstring>("misc", transcode(_("Miscellaneous"))));
+  task_section_descriptions.push_back(pair<string,wstring>("unknown", transcode(_("Unrecognized tasks"))));
 }
 
 /** Uses the fact that 0<=Relevance<=10 to encode Relevance as a character. */
