@@ -1,6 +1,6 @@
 // ui.cc
 //
-//   Copyright 2000-2006 Daniel Burrows <dburrows@debian.org>
+//   Copyright 2000-2007 Daniel Burrows <dburrows@debian.org>
 //
 //   This program is free software; you can redistribute it and/or
 //   modify it under the terms of the GNU General Public License as
@@ -40,6 +40,7 @@
 #include <sys/types.h>
 
 #include <fstream>
+#include <sstream>
 #include <utility>
 
 #ifdef HAVE_CONFIG_H
@@ -157,6 +158,7 @@ sigc::signal0<bool, accumulate_or> package_changelog_enabled;
 sigc::signal0<bool, accumulate_or> find_search_enabled;
 sigc::signal0<bool, accumulate_or> find_search_back_enabled;
 sigc::signal0<bool, accumulate_or> find_research_enabled;
+sigc::signal0<bool, accumulate_or> find_repeat_search_back_enabled;
 sigc::signal0<bool, accumulate_or> find_limit_enabled;
 sigc::signal0<bool, accumulate_or> find_cancel_limit_enabled;
 sigc::signal0<bool, accumulate_or> find_broken_enabled;
@@ -182,6 +184,7 @@ sigc::signal0<bool, accumulate_or> resolver_view_target_enabled;
 sigc::signal0<bool, accumulate_or> find_search;
 sigc::signal0<bool, accumulate_or> find_search_back;
 sigc::signal0<bool, accumulate_or> find_research;
+sigc::signal0<bool, accumulate_or> find_repeat_search_back;
 sigc::signal0<bool, accumulate_or> find_limit;
 sigc::signal0<bool, accumulate_or> find_cancel_limit;
 sigc::signal0<bool, accumulate_or> find_broken;
@@ -250,6 +253,11 @@ static void pager_repeat_search(vs_pager &p)
   p.search_for(L"");
 }
 
+static void pager_repeat_search_back(vs_pager &p)
+{
+  p.search_back_for(L"");
+}
+
 static vs_widget_ref make_error_dialog(const vs_text_layout_ref &layout)
 {
   vs_table_ref t=vs_table::create();
@@ -400,6 +408,29 @@ static void do_su_to_root(string args)
       return;
     }
 
+  std::string root_command = aptcfg->Find(PACKAGE "::Get-Root-Command",
+					  "su:/bin/su");
+
+  if(root_command == "su")
+    root_command = "su:/bin/su";
+  else if(root_command == "sudo")
+    root_command = "sudo:/usr/bin/sudo";
+
+  std::string::size_type splitloc = root_command.find(':');
+  if(splitloc == std::string::npos)
+    {
+      _error->Error(_("Invalid Get-Root-Command; it should start with su: or sudo:"));
+      return;
+    }
+
+  std::string protocol(root_command, 0, splitloc);
+  if(protocol != "su" && protocol != "sudo")
+    {
+      _error->Error(_("Invalid Get-Root-Command; it should start with su: or sudo:, not %s:"), protocol.c_str());
+      return;
+    }
+  std::string root_program(root_command, splitloc + 1);
+
   temp::dir  tempdir("aptitude");
   temp::name statusname(tempdir, "pkgstates");
   temp::name fifoname(tempdir, "control");
@@ -411,10 +442,23 @@ static void do_su_to_root(string args)
       return;
     }
 
+  // Shut curses down.  This is done before we fork because otherwise
+  // we'll end up with curses apparently being initialized in the
+  // child and with mutexes locked (despite the fact that no thread
+  // holds them), and get horribly confused, especially if we try to
+  // invoke exit(3).
+  //
+  // An alternative is to invoke _exit(2) instead, but it seems
+  // cleaner to suspend curses right here.
+  vscreen_suspend();
+
   int pid=fork();
 
   if(pid==-1)
-    _error->Error("Unable to fork: %s", strerror(errno));
+    {
+      _error->Error("Unable to fork: %s", strerror(errno));
+      vscreen_resume();
+    }
   else if(pid==0) // I'm a child!
     {
       // Read one byte from the FIFO for synchronization
@@ -430,10 +474,57 @@ static void do_su_to_root(string args)
       //
       // What happens if tmpdir has spaces in it?  Can I get more
       // control over how the su-to-root function is called?
-      char cmdbuf[512];
-      snprintf(cmdbuf, 512, "%s -S %s %s",
-	       argv0, statusname.get_name().c_str(), args.c_str());
-      execl("/bin/su", "/bin/su", "-c", cmdbuf, NULL);
+      if(protocol == "su")
+	{
+	  std::ostringstream cmdbuf;
+	  cmdbuf << argv0 << " -S "
+		 << statusname.get_name() << " "
+		 << args;
+	  execl(root_program.c_str(), root_program.c_str(), "-c", cmdbuf.str().c_str(), NULL);
+
+	  exit(1);
+	}
+      else if(protocol == "sudo")
+	{
+	  std::vector<std::string> cmdlist;
+	  // Split whitespace in the input command.
+	  std::string command = root_program + " " + argv0 + " -S " + statusname.get_name() + " " + args;
+	  std::string::const_iterator it = command.begin();
+	  while(it != command.end() && isspace(*it))
+	    ++it;
+
+	  while(it != command.end())
+	    {
+	      std::string tmp;
+	      while(it != command.end() && !isspace(*it))
+		{
+		  tmp += *it;
+		  ++it;
+		}
+
+	      cmdlist.push_back(tmp);
+
+	      while(it != command.end() && isspace(*it))
+		++it;
+	    }
+
+	  const char **real_cmd = new const char*[cmdlist.size() + 1];
+	  for(std::string::size_type i = 0; i < cmdlist.size(); ++i)
+	    real_cmd[i] = cmdlist[i].c_str();
+	  real_cmd[cmdlist.size()] = NULL;
+
+	  execv(real_cmd[0], const_cast<char* const*>(real_cmd));
+
+	  std::string errmsg = ssprintf("aptitude: do_su_to_root: exec \"%s\" failed", root_program.c_str());
+	  perror(errmsg.c_str());
+
+	  delete[] real_cmd;
+
+	  // Eek, something bad happened.
+	  exit(1);
+	}
+      else
+	exit(1); // Should never happen -- see the test above.
     }
   else
     {
@@ -445,9 +536,6 @@ static void do_su_to_root(string args)
       if(apt_cache_file != NULL)
 	(*apt_cache_file)->save_selection_list(foo, statusname.get_name().c_str());
 
-      // Shut curses down.
-      vscreen_suspend();
-
       // Ok, wake the other process up.
       char tmp=0;
       int fd=open(fifoname.get_name().c_str(), O_WRONLY);
@@ -519,10 +607,34 @@ static void do_hide_reload_message()
     }
 }
 
+/** \brief If this is \b true, there's a "really quit aptitude?"
+ *  prompt displayed.
+ *
+ *  The sole purpose of this variable is to prevent the dialog
+ *  box that asks about quitting from showing up multiple times.
+ */
+static bool really_quit_active = false;
+
+static void do_really_quit_answer(bool should_i_quit)
+{
+  really_quit_active = false;
+
+  if(should_i_quit)
+    file_quit();
+}
+
 static void do_quit()
 {
   if(aptcfg->FindB(PACKAGE "::UI::Prompt-On-Exit", true))
-    prompt_yesno(_("Really quit Aptitude?"), false, arg(file_quit.make_slot()), NULL);
+    {
+      if(!really_quit_active)
+	{
+	  really_quit_active = true;
+	  prompt_yesno(_("Really quit Aptitude?"), false,
+		       arg(sigc::bind(ptr_fun(do_really_quit_answer), true)),
+		       arg(sigc::bind(ptr_fun(do_really_quit_answer), false)));
+	}
+    }
   else
     file_quit();
 }
@@ -849,7 +961,8 @@ static void do_help_license()
   vs_widget_ref w=vs_dialog_fileview(HELPDIR "/COPYING",
 				     NULL,
 				     arg(sigc::ptr_fun(pager_search)),
-				     arg(sigc::ptr_fun(pager_repeat_search)));
+				     arg(sigc::ptr_fun(pager_repeat_search)),
+				     arg(sigc::ptr_fun(pager_repeat_search_back)));
   w->show_all();
 
   popup_widget(w);
@@ -873,6 +986,7 @@ static void do_help_help()
   vs_widget_ref w=vs_dialog_fileview(buf, NULL,
 				     arg(sigc::ptr_fun(pager_search)),
 				     arg(sigc::ptr_fun(pager_repeat_search)),
+				     arg(sigc::ptr_fun(pager_repeat_search_back)),
 				     encoding);
   w->show_all();
 
@@ -906,6 +1020,8 @@ static void do_help_readme()
 		 sigc::bind(sigc::ptr_fun(&pager_search), p.weak_ref()));
   p->connect_key("ReSearch", &global_bindings,
 		 sigc::bind(sigc::ptr_fun(&pager_repeat_search), p.weak_ref()));
+  p->connect_key("RepeatSearchBack", &global_bindings,
+		 sigc::bind(sigc::ptr_fun(&pager_repeat_search_back), p.weak_ref()));
 
   t->add_widget_opts(p, 0, 0, 1, 1,
 		     vs_table::EXPAND | vs_table::FILL | vs_table::SHRINK,
@@ -1198,13 +1314,11 @@ static void do_keep_all()
 {
   auto_ptr<undo_group> undo(new apt_undo_group);
 
-  (*apt_cache_file)->begin_action_group();
+  aptitudeDepCache::action_group group(*apt_cache_file, undo.get());
 
   for(pkgCache::PkgIterator i=(*apt_cache_file)->PkgBegin();
       !i.end(); ++i)
-    (*apt_cache_file)->mark_keep(i, true, false, undo.get());
-
-  (*apt_cache_file)->end_action_group(undo.get());
+    (*apt_cache_file)->mark_keep(i, false, false, undo.get());
 
   if(!undo.get()->empty())
     apt_undos->add_item(undo.release());
@@ -1635,6 +1749,30 @@ class interactive_continuation : public resolver_manager::background_continuatio
       manager->state_changed();
     }
   };
+
+  class solution_search_aborted_event : public vscreen_event
+  {
+    resolver_manager *manager;
+    std::string msg;
+  public:
+    solution_search_aborted_event(resolver_manager *_manager,
+				  std::string _msg)
+      : manager(_manager), msg(_msg)
+    {
+    }
+
+    void dispatch()
+    {
+      resolver_manager::state st = manager->state_snapshot();
+
+      if(st.selected_solution == st.generated_solutions)
+	manager->select_previous_solution();
+      show_message(ssprintf("Fatal error in dependency resolver.  You can continue searching, but some solutions might be impossible to generate.\n\n%s",
+			    msg.c_str()));
+
+      manager->state_changed();
+    }
+  };
 public:
   interactive_continuation(resolver_manager *_manager)
     :manager(_manager)
@@ -1659,6 +1797,11 @@ public:
   void interrupted()
   {
   }
+
+  void aborted(const Exception &e)
+  {
+    vscreen_post_event(new solution_search_aborted_event(manager, e.errmsg()));
+  }
 };
 
 // If the current solution pointer is past the end of the solution
@@ -1674,7 +1817,8 @@ static void start_solution_calculation(bool blocking)
   if(state.resolver_exists &&
      state.selected_solution == state.generated_solutions &&
      !state.solutions_exhausted &&
-     !state.background_thread_active)
+     !state.background_thread_active &&
+     !state.background_thread_aborted)
     {
       const int selected = state.selected_solution;
       const int limit = aptcfg->FindI(PACKAGE "::ProblemResolver::StepLimit", 5000);
@@ -1714,7 +1858,12 @@ void do_next_solution()
   if(!do_next_solution_enabled())
     beep();
   else
-    resman->select_next_solution();
+    {
+      // If an error was encountered, pressing "next solution"
+      // skips it.
+      resman->discard_error_information();
+      resman->select_next_solution();
+    }
 }
 
 static bool do_previous_solution_enabled()
@@ -2090,6 +2239,10 @@ vs_menu_info search_menu[]={
 	       N_("Repeat the last search"),
 	       sigc::hide_return(find_research.make_slot()),
 	       find_research_enabled.make_slot()),
+  vs_menu_info(vs_menu_info::VS_MENU_ITEM, N_("Find Again ^Backwards"), "RepeatSearchBack",
+	       N_("Repeat the last search in the opposite direction"),
+	       sigc::hide_return(find_repeat_search_back.make_slot()),
+	       find_repeat_search_back_enabled.make_slot()),
   VS_MENU_SEPARATOR,
   vs_menu_info(vs_menu_info::VS_MENU_ITEM, N_("^Limit Display"),
 	       "ChangePkgTreeLimit", N_("Apply a filter to the package list"),
@@ -2677,6 +2830,7 @@ void prompt_string(const std::wstring &prompt,
   if(aptcfg->FindB(PACKAGE "::UI::Minibuf-Prompts"))
     {
       vs_editline_ref e=vs_editline::create(prompt, text, history);
+      e->set_clear_on_first_edit(true);
       if(slot)
 	e->entered.connect(*slot);
 
