@@ -1,6 +1,6 @@
 // view_changelog.cc
 //
-//   Copyright (C) 2004-2005 Daniel Burrows
+//   Copyright (C) 2004-2005, 2007 Daniel Burrows
 //
 //   This program is free software; you can redistribute it and/or
 //   modify it under the terms of the GNU General Public License as
@@ -53,7 +53,7 @@ class pkg_changelog_screen : public vs_file_pager, public menu_redirect
     last_search_forwards = true;
 
     prompt_string(transcode(_("Search for: ")),
-		  L"",
+		  get_last_search(),
 		  arg(sigc::mem_fun(*this, &vs_pager::search_for)),
 		  NULL,
 		  NULL,
@@ -65,7 +65,7 @@ class pkg_changelog_screen : public vs_file_pager, public menu_redirect
     last_search_forwards = false;
 
     prompt_string(transcode(_("Search backwards for: ")),
-		  L"",
+		  get_last_search(),
 		  arg(sigc::mem_fun(*this, &vs_pager::search_back_for)),
 		  NULL,
 		  NULL,
@@ -80,6 +80,14 @@ class pkg_changelog_screen : public vs_file_pager, public menu_redirect
       search_back_for(L"");
   }
 
+  void do_repeat_search_back()
+  {
+    if(!last_search_forwards)
+      search_for(L"");
+    else
+      search_back_for(L"");
+  }
+
 protected:
   pkg_changelog_screen(const temp::name &filename,
 		       int x = 0, int y = 0,
@@ -92,6 +100,8 @@ protected:
 		sigc::mem_fun(*this, &pkg_changelog_screen::do_search_back));
     connect_key("ReSearch", &global_bindings,
 		sigc::mem_fun(*this, &pkg_changelog_screen::do_repeat_search));
+    connect_key("RepeatSearchBack", &global_bindings,
+		sigc::mem_fun(*this, &pkg_changelog_screen::do_repeat_search_back));
   }
 
 public:
