@@ -1,6 +1,21 @@
 // vs_editline.cc
 //
-//  Copyright 2000 Daniel Burrows
+//   Copyright (C) 2000, 2007 Daniel Burrows
+//
+//   This program is free software; you can redistribute it and/or
+//   modify it under the terms of the GNU General Public License as
+//   published by the Free Software Foundation; either version 2 of
+//   the License, or (at your option) any later version.
+//
+//   This program is distributed in the hope that it will be useful,
+//   but WITHOUT ANY WARRANTY; without even the implied warranty of
+//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+//   General Public License for more details.
+//
+//   You should have received a copy of the GNU General Public License
+//   along with this program; see the file COPYING.  If not, write to
+//   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+//   Boston, MA 02111-1307, USA.
 
 #include "vs_editline.h"
 
@@ -19,7 +34,7 @@ vs_editline::vs_editline(const string &_prompt, const string &_text,
 			 history_list *_history)
   :vscreen_widget(), curloc(_text.size()),
    startloc(0), desired_size(-1), history(_history),
-   history_loc(0), using_history(false)
+   history_loc(0), using_history(false), clear_on_first_edit(false)
 {
   // Just spew a partial/null string if errors happen for now.
   transcode(_prompt.c_str(), prompt);
@@ -36,7 +51,7 @@ vs_editline::vs_editline(const wstring &_prompt, const wstring &_text,
 			 history_list *_history)
   :vscreen_widget(), prompt(_prompt), text(_text), curloc(_text.size()),
    startloc(0), desired_size(-1), history(_history),
-   history_loc(0), using_history(false)
+   history_loc(0), using_history(false), clear_on_first_edit(false)
 {
   set_bg_style(get_style("EditLine"));
 
@@ -49,7 +64,7 @@ vs_editline::vs_editline(int maxlength, const string &_prompt,
 			 const string &_text, history_list *_history)
   :vscreen_widget(), curloc(0),
    startloc(0), desired_size(maxlength), history(_history), history_loc(0),
-   using_history(false)
+   using_history(false), clear_on_first_edit(false)
 {
   // As above, ignore errors.
   transcode(_prompt, prompt);
@@ -63,7 +78,7 @@ vs_editline::vs_editline(int maxlength, const wstring &_prompt,
 			 const wstring &_text, history_list *_history)
   :vscreen_widget(), prompt(_prompt), text(_text), curloc(0),
    startloc(0), desired_size(maxlength), history(_history), history_loc(0),
-   using_history(false)
+   using_history(false), clear_on_first_edit(false)
 {
   set_bg_style(get_style("EditLine"));
   do_layout.connect(sigc::mem_fun(*this, &vs_editline::normalize_cursor));
@@ -181,6 +196,9 @@ bool vs_editline::handle_key(const key &k)
 {
   vs_widget_ref tmpref(this);
 
+  bool clear_on_this_edit = clear_on_first_edit;
+  clear_on_first_edit = false;
+
   if(bindings->key_matches(k, "DelBack"))
     {
       if(curloc>0)
@@ -350,6 +368,13 @@ bool vs_editline::handle_key(const key &k)
     return false;
   else
     {
+      if(clear_on_this_edit)
+	{
+	  text.clear();
+	  curloc = 0;
+	  startloc = 0;
+	}
+
       text.insert(curloc++, 1, k.ch);
       normalize_cursor();
       text_changed(wstring(text));
@@ -386,6 +411,9 @@ void vs_editline::dispatch_mouse(short id, int x, int y, int z, mmask_t bstate)
   vs_widget_ref tmpref(this);
 
   size_t mouseloc=startloc; // The character at which the mouse press occured
+
+  clear_on_first_edit = false;
+
   while(mouseloc<prompt.size()+text.size() && x>0)
     {
       int curwidth=wcwidth(get_char(mouseloc));
