@@ -14,7 +14,7 @@ class vs_passthrough:public vs_container
   void lost_focus();
 
 protected:
-  virtual bool handle_char(chtype ch);
+  virtual bool handle_key(const key &k);
 
   // These call focussed() and unfocussed() on the result of get_focus().
   // (convenience methods)
@@ -22,11 +22,16 @@ protected:
   // Provided to make it easier to manage focus simply.
   void defocus();
   void refocus();
-public:
+
+protected:
   vs_passthrough();
 
+public:
   // Returns the currently focussed widget, if any.
-  virtual vscreen_widget *get_focus()=0;
+  virtual vs_widget_ref get_focus()=0;
+
+  vs_widget_ref get_active_widget();
+
   virtual void dispatch_mouse(short id, int x, int y, int z, mmask_t bstate);
 
   virtual bool focus_me();
@@ -34,4 +39,6 @@ public:
   virtual point get_cursorloc();
 };
 
+typedef ref_ptr<vs_passthrough> vs_passthrough_ref;
+
 #endif
