@@ -1,6 +1,6 @@
 // vscreen.cc
 //
-//  Copyright 1999-2005 Daniel Burrows
+//  Copyright 1999-2005, 2007 Daniel Burrows
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -43,11 +43,13 @@
 #include "config/style.h"
 
 #include <generic/util/event_queue.h>
+#include <generic/util/util.h>
 #include <generic/util/threads.h>
 
 // For _()
 #include "../aptitude.h"
 
+#include <limits.h>
 #include <signal.h>
 
 #include <generic/util/eassert.h>
@@ -201,55 +203,135 @@ void vscreen_post_event(vscreen_event *ev)
 //////////////////////////////////////////////////////////////////////
 // Event management threads
 
-/** This thread is responsible for posting wget_wch() calls. */
+/** This thread is responsible for posting wget_wch() calls.
+ *
+ *  Note that the actual call to wget_wch must take place in the
+ *  foreground thread, because wget_wch will invoke wrefresh().
+ *  So instead of calling it in the background thread, I post
+ *  input events to the foreground thread.
+ *
+ *  To prevent the background thread from spamming the foreground
+ *  thread with events, I suspend it until the event actually
+ *  triggers.
+ */
 class input_thread
 {
-  class key_input_event : public vscreen_event
+  class get_input_event : public vscreen_event
   {
-    key k;
+    // A reference to the parent's condition mutex;
+    // should be held while we signal the condition.
+    threads::mutex &m;
+    // A reference to the parent's variable indicating
+    // whether the event has triggered.  Will be set
+    // to "true" after we try to read all available
+    // keystrokes.
+    bool &b;
+    // A reference to the parent's condition variable.
+    threads::condition &c;
+
   public:
-    key_input_event (const key &_k)
-      :k(_k)
+    get_input_event(threads::mutex &_m, bool &_b, threads::condition &_c)
+      : m(_m), b(_b), c(_c)
     {
     }
 
     void dispatch()
     {
-      if(global_bindings.key_matches(k, "Refresh"))
-	vscreen_redraw();
-      else
-	toplevel->dispatch_key(k);
+      // NB: use the GLOBAL getch function to avoid weirdness
+      // referencing the state of toplevel.
+      wint_t wch = 0;
+      int status;
+
+      bool done = false;
+
+      while(!done)
+	{
+	  // I assume here that vscreen_init() set nodelay.
+	  do
+	    {
+	      status = get_wch(&wch);
+	    } while(status == KEY_CODE_YES && wch == KEY_RESIZE);
+
+	  if(status == ERR) // No more to read.
+	    {
+	      threads::mutex::lock l(m);
+	      b = true;
+	      c.wake_all();
+	      done = true;
+	    }
+	  else
+	    {
+	      key k(wch, status == KEY_CODE_YES);
+
+	      if(wch == KEY_MOUSE)
+		{
+		  if(toplevel.valid())
+		    {
+		      MEVENT ev;
+		      getmouse(&ev);
+
+		      toplevel->dispatch_mouse(ev.id, ev.x, ev.y, ev.z, ev.bstate);
+		    }
+		}
+	      else
+		{
+		  if(global_bindings.key_matches(k, "Refresh"))
+		    vscreen_redraw();
+		  else
+		    toplevel->dispatch_key(k);
+		}
+	    }
+	}
     }
   };
 
-  class mouse_input_event : public vscreen_event
+  class fatal_input_exception : public Exception
   {
-    MEVENT ev;
-
+    int err;
   public:
-    mouse_input_event(const MEVENT &_ev)
-      :ev(_ev)
+    fatal_input_exception(int _err)
+      : err(_err)
+    {
+    }
+
+    std::string errmsg() const
+    {
+      return "Unable to read from stdin: " + sstrerror(err);
+    }
+  };
+
+  class fatal_input_error : public vscreen_event
+  {
+    int err;
+  public:
+    fatal_input_error(int _err)
+      : err(_err)
     {
     }
 
     void dispatch()
     {
-      if(toplevel.valid())
-	toplevel->dispatch_mouse(ev.id, ev.x, ev.y, ev.z, ev.bstate);
+      throw new fatal_input_exception(err);
     }
   };
 
+  threads::mutex input_event_mutex;
+  // Used to block this thread until an event to read input fires.
+  bool input_event_fired;
+  threads::condition input_event_condition;
+
   static input_thread instance;
 
   static threads::mutex instance_mutex;
   static threads::thread *instancet;
+
 public:
   static void start()
   {
     threads::mutex::lock l(instance_mutex);
 
     if(instancet == NULL)
-      instancet = new threads::thread(instance);
+      instancet = new threads::thread(threads::make_bootstrap_proxy(&instance));
   }
 
   static void stop()
@@ -265,34 +347,50 @@ public:
       }
   }
 
-  void operator()() const
+  void operator()()
   {
-    // NB: use the GLOBAL getch function to avoid weirdness
-    // referencing the state of toplevel.
+    threads::mutex::lock l(input_event_mutex);
+    input_event_fired = false;
+
+    // Important note: this routine only blocks indefinitely in
+    // select() and pthread_cond_wait(), assuming no bugs in
+    // vscreen_post_event().  pthread_cond_wait() is a cancellation
+    // point.  select() should be but isn't, but there is a
+    // workaround below.
+
     while(1)
       {
-	wint_t wch = 0;
-	int status;
+	// Select on stdin; when we see that input is available, spawn
+	// an input event.
+	fd_set selectfds;
+	FD_ZERO(&selectfds);
+	FD_SET(0, &selectfds);
 
-	do
+	pthread_testcancel();
+	int result = select(1, &selectfds, NULL, NULL, NULL);
+	pthread_testcancel();	// Workaround for Linux threads suckage.
+                                // See pthread_cancel(3).
+
+	if(result != 1)
 	  {
-	    status = get_wch(&wch);
-	  } while(status == KEY_CODE_YES && wch == KEY_RESIZE);
-
-	key k(wch, status == KEY_CODE_YES);
-
-	if(status == ERR)
-	  return; // ???
-
-	if(wch == KEY_MOUSE)
-	  {
-	    MEVENT ev;
-	    getmouse(&ev);
-
-	    vscreen_post_event(new mouse_input_event(ev));
+	    if(errno != EINTR)
+	      {
+		// Probably means that there was an error reading
+		// standard input.  (could also be ENOMEM)
+		vscreen_post_event(new fatal_input_error(errno));
+		break;
+	      }
 	  }
 	else
-	  vscreen_post_event(new key_input_event(k));
+	  {
+	    vscreen_post_event(new get_input_event(input_event_mutex,
+						   input_event_fired,
+						   input_event_condition));
+
+	    while(!input_event_fired)
+	      input_event_condition.wait(l);
+	    input_event_fired = false;
+	  }
       }
   }
 };
@@ -431,7 +529,7 @@ class timeout_thread
 	gettimeofday(&curtime, 0);
 
 	if(timeval_subtract(&result, &i->second.activate_time, &curtime) == 1 ||
-	   result.tv_sec == 0 && result.tv_usec <= 10)
+	   (result.tv_sec == 0 && result.tv_usec <= 10))
 	  {
 	    vscreen_post_event(i->second.ev);
 	    timeouts.erase(i);
@@ -461,7 +559,7 @@ class timeout_thread
 	j++;
 	timeval diff;
 	if(timeval_subtract(&diff, &i->second.activate_time, &curtime) == 1 ||
-	   diff.tv_sec == 0 && diff.tv_usec <= 10)
+	   (diff.tv_sec == 0 && diff.tv_usec <= 10))
 	  {
 	    tv_out = curtime;
 	    return true;
@@ -497,23 +595,6 @@ class timeout_thread
   // The global instance; this is a Singleton.
   static timeout_thread instance;
 
-  // Unfortunately, technical considerations in the threading code
-  // mean that the actual thread object is expected to be copyable.
-  // Hence this proxy:
-  class timeout_proxy
-  {
-    timeout_thread &real_thread;
-  public:
-    timeout_proxy(timeout_thread &_real_thread)
-      : real_thread(_real_thread)
-    {
-    }
-
-    void operator()() const
-    {
-      real_thread();
-    }
-  };
 public:
   static timeout_thread &get_instance()
   {
@@ -531,7 +612,8 @@ public:
 	throw SingletonViolationException();
       }
 
-    instance.running_thread.put(new threads::thread(timeout_proxy(instance)));
+    threads::thread *t = new threads::thread(threads::make_bootstrap_proxy(&instance));
+    instance.running_thread.put(t);
   }
 
   static void stop()
@@ -626,6 +708,8 @@ timeout_thread timeout_thread::instance;
 
 void vscreen_init()
 {
+  threads::mutex::lock l(vscreen_get_mutex());
+
   keybinding upkey, downkey, leftkey, rightkey, quitkey, homekey, endkey;
   keybinding historynextkey, historyprevkey;
   keybinding delfkey, delbkey, ppagekey, npagekey;
@@ -687,6 +771,7 @@ void vscreen_init()
 
   curses_avail=true;
   cbreak();
+  rootwin.nodelay(true);
   rootwin.keypad(true);
 
   global_bindings.set("Quit", quitkey);
@@ -709,6 +794,7 @@ void vscreen_init()
   global_bindings.set("Search", key(L'/', false));
   global_bindings.set("SearchBack", key(L'\\', false));
   global_bindings.set("ReSearch", researchkey);
+  global_bindings.set("RepeatSearchBack", key(L'N', false));
   global_bindings.set("DelBack", delbkey);
   global_bindings.set("DelForward", delfkey);
 
@@ -797,6 +883,8 @@ void vscreen_install_sighandlers()
 
 void vscreen_handleresize()
 {
+  threads::mutex::lock l(vscreen_get_mutex());
+
   toplevel->set_owner_window(NULL, 0, 0, 0, 0);
   resize();
   toplevel->set_owner_window(rootwin, 0, 0, rootwin.getmaxx(), rootwin.getmaxy());
@@ -821,6 +909,8 @@ void vscreen_updatecursor()
 
 void vscreen_updatecursornow()
 {
+  threads::mutex::lock(vscreen_get_mutex());
+
   if(toplevel->get_cursorvisible())
     {
       point p=toplevel->get_cursorloc();
@@ -844,6 +934,8 @@ void vscreen_update()
 
 void vscreen_updatenow()
 {
+  threads::mutex::lock l(vscreen_get_mutex());
+
   if(toplevel.valid())
     {
       toplevel->display(get_style("Default"));
@@ -864,12 +956,16 @@ void vscreen_queuelayout()
 
 void vscreen_layoutnow()
 {
+  threads::mutex::lock l(vscreen_get_mutex());
+
   toplevel->do_layout();
 }
 
 void vscreen_tryupdate()
 {
-  threads::mutex::lock l(pending_updates_mutex);
+  threads::mutex::lock l(vscreen_get_mutex());
+
+  threads::mutex::lock l2(pending_updates_mutex);
 
   update_state needs = pending_updates;
 
@@ -891,6 +987,8 @@ void vscreen_tryupdate()
 
 bool vscreen_poll()
 {
+  threads::mutex::lock l(vscreen_get_mutex());
+
   bool rval=false;
 
   vscreen_event *ev = NULL;
@@ -947,6 +1045,8 @@ void vscreen_exitmain()
 
 void vscreen_suspend_without_signals()
 {
+  threads::mutex::lock l(vscreen_get_mutex());
+
   input_thread::stop();
   signal_thread::stop();
   timeout_thread::stop();
@@ -963,6 +1063,8 @@ void vscreen_suspend_without_signals()
 
 void vscreen_suspend()
 {
+  threads::mutex::lock l(vscreen_get_mutex());
+
   suspended_with_signals = true;
 
   struct sigaction act;
@@ -979,6 +1081,8 @@ void vscreen_suspend()
 
 void vscreen_shutdown()
 {
+  threads::mutex::lock l(vscreen_get_mutex());
+
   if(toplevel.valid())
     toplevel->destroy();
   toplevel = NULL;
@@ -993,6 +1097,8 @@ void vscreen_shutdown()
 
 void vscreen_resume()
 {
+  threads::mutex::lock l(vscreen_get_mutex());
+
   if(suspended_with_signals)
     {
       sigaction(SIGCONT, &oldsigcont, NULL);
@@ -1018,7 +1124,9 @@ void vscreen_resume()
 
 void vscreen_redraw()
 {
-  threads::mutex::lock l(pending_updates_mutex);
+  threads::mutex::lock l(vscreen_get_mutex());
+
+  threads::mutex::lock l2(pending_updates_mutex);
 
   if(toplevel.valid())
     {
